==> firebase_pagination-0.0.1-dev.tar.gz: lib/src/firestore_pagination.dart <==
// Flutter Packages
import 'package:flutter/material.dart';

// Dart Packages
import 'dart:async';

// Firebase Packages
import 'package:cloud_firestore/cloud_firestore.dart';

// Data Models
import 'models/view_type.dart';
import 'models/wrap_options.dart';

// Widgets
import 'widgets/build_pagination.dart';

// Functions
import 'functions/fetch_documents.dart';

class FirestorePagination extends StatefulWidget {
  const FirestorePagination({
    Key? key,
    required this.query,
    required this.itemBuilder,
    this.startAfter,
    this.limit = 10,
    this.viewType = ViewType.list,
    this.isRealtime = true,
    this.gridDelegate = const SliverGridDelegateWithFixedCrossAxisCount(
      crossAxisCount: 2,
      childAspectRatio: 1,
    ),
    this.wrapOptions = const WrapOptions(),
  }) : super(key: key);

  final Query query;
  final Widget Function(BuildContext, DocumentSnapshot, int) itemBuilder;
  final DocumentSnapshot? startAfter;
  final int limit;
  final ViewType viewType;
  final bool isRealtime;
  final SliverGridDelegate gridDelegate;
  final WrapOptions wrapOptions;

  @override
  State<FirestorePagination> createState() => _FirestorePaginationState();
}

class _FirestorePaginationState extends State<FirestorePagination> {
  List<DocumentSnapshot> _documents = [];
  StreamSubscription<QuerySnapshot>? _streamSub;
  ScrollController controller = ScrollController();
  bool isLoading = true;
  bool isFetching = false;

  Future<void> _loadDocuments() async {
    setState(() {
      isFetching = true;
    });

    final List<DocumentSnapshot> documents = await fetchDocuments(
      query: widget.query,
      limit: widget.limit,
      startAfter: widget.startAfter ?? _documents.last,
    );

    _documents += documents;

    isFetching = false;
    isLoading = false;

    if (mounted) {
      setState(() {});
    }

    _setAutoUpdateListener();
  }

  void _setAutoUpdateListener() async {
    await _streamSub?.cancel();
    _streamSub = widget.query
        .limit(_documents.length + (widget.isRealtime ? 1 : 0))
        .snapshots()
        .listen(
      (QuerySnapshot snapshot) {
        _documents = snapshot.docs;
        if (mounted) {
          setState(() {});
        }
        _setAutoUpdateListener();
      },
    );
  }

  void _scrollListener() {
    final ScrollPosition position = controller.position;
    if (position.pixels >= position.maxScrollExtent &&
        !isLoading &&
        !isFetching) {
      _loadDocuments();
    }
  }

  @override
  void initState() {
    super.initState();
    _loadDocuments();

    controller.addListener(_scrollListener);
  }

  @override
  void dispose() {
    _streamSub?.cancel();
    controller.removeListener(_scrollListener);
    controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return isLoading
        ? const Center(child: CircularProgressIndicator.adaptive())
        : BuildPagination(
            controller: controller,
            items: _documents,
            itemBuilder: widget.itemBuilder as dynamic,
            isLoading: isLoading,
          );
  }
}

==> firebase_pagination-0.0.2-dev.tar.gz: lib/src/firestore_pagination.dart <==
// Flutter Packages
import 'package:flutter/material.dart';

// Dart Packages
import 'dart:async';

// Firebase Packages
import 'package:cloud_firestore/cloud_firestore.dart';

// Data Models
import 'models/view_type.dart';
import 'models/wrap_options.dart';

// Widgets
import 'widgets/defaults/bottom_loader.dart';
import 'widgets/defaults/empty_screen.dart';
import 'widgets/defaults/initial_loader.dart';
import 'widgets/views/build_pagination.dart';

// Functions
import 'functions/fetch_documents.dart';

class FirestorePagination extends StatefulWidget {
  const FirestorePagination({
    Key? key,
    required this.query,
    required this.itemBuilder,
    this.startAfter,
    this.limit = 10,
    this.viewType = ViewType.list,
    this.isRealtime = true,
    this.gridDelegate = const SliverGridDelegateWithFixedCrossAxisCount(
      crossAxisCount: 2,
      childAspectRatio: 1,
    ),
    this.wrapOptions = const WrapOptions(),
    this.onEmpty = const EmptyScreen(),
    this.bottomLoader = const BottomLoader(),
    this.initialLoader = const InitialLoader(),
  }) : super(key: key);

  final Query query;
  final Widget Function(BuildContext, DocumentSnapshot, int) itemBuilder;
  final DocumentSnapshot? startAfter;
  final int limit;
  final ViewType viewType;
  final bool isRealtime;
  final SliverGridDelegate gridDelegate;
  final WrapOptions wrapOptions;
  final Widget onEmpty;
  final Widget bottomLoader;
  final Widget initialLoader;

  @override
  State<FirestorePagination> createState() => _FirestorePaginationState();
}

class _FirestorePaginationState extends State<FirestorePagination> {
  List<DocumentSnapshot> _documents = [];
  StreamSubscription<QuerySnapshot>? _streamSub;
  ScrollController controller = ScrollController();
  bool isLoading = true;
  bool isFetching = false;

  Future<void> _loadDocuments() async {
    setState(() {
      isFetching = true;
    });

    final List<DocumentSnapshot> documents = await fetchDocuments(
      query: widget.query,
      limit: widget.limit,
      startAfter: widget.startAfter ?? _documents.last,
    );

    _documents += documents;

    isFetching = false;
    isLoading = false;

    if (mounted) {
      setState(() {});
    }

    _setAutoUpdateListener();
  }

  void _setAutoUpdateListener() async {
    await _streamSub?.cancel();
    _streamSub = widget.query
        .limit(_documents.length + (widget.isRealtime ? 1 : 0))
        .snapshots()
        .listen(
      (QuerySnapshot snapshot) {
        _documents = snapshot.docs;
        if (mounted) {
          setState(() {});
        }
        _setAutoUpdateListener();
      },
    );
  }

  void _scrollListener() {
    final ScrollPosition position = controller.position;
    if (position.pixels >= position.maxScrollExtent &&
        !isLoading &&
        !isFetching) {
      _loadDocuments();
    }
  }

  @override
  void initState() {
    super.initState();
    _loadDocuments();

    controller.addListener(_scrollListener);
  }

  @override
  void dispose() {
    _streamSub?.cancel();
    controller.removeListener(_scrollListener);
    controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return isLoading
        ? widget.initialLoader
        : _documents.isEmpty
            ? widget.onEmpty
            : BuildPagination(
                controller: controller,
                items: _documents,
                itemBuilder: widget.itemBuilder,
                isLoading: isLoading,
                bottomLoader: widget.bottomLoader,
                gridDelegate: widget.gridDelegate,
                viewType: widget.viewType,
                wrapOptions: widget.wrapOptions,
              );
  }
}
